"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const simpleProject_1 = require("@volar/language-server/lib/project/simpleProject");
const node_1 = require("@volar/language-server/node");
const language_core_1 = require("@vue/language-core");
const language_service_1 = require("@vue/language-service");
const vscode_uri_1 = require("vscode-uri");
const connection = (0, node_1.createConnection)();
const server = (0, node_1.createServer)(connection);
connection.listen();
connection.onInitialize(params => {
    const options = params.initializationOptions;
    if (!options.typescript?.tsdk) {
        throw new Error('typescript.tsdk is required');
    }
    if (!options.typescript?.tsserverRequestCommand) {
        connection.console.warn('typescript.tsserverRequestCommand is required since >= 3.0 for complete TS features');
    }
    const { typescript: ts } = (0, node_1.loadTsdkByPath)(options.typescript.tsdk, params.locale);
    const tsconfigProjects = (0, language_service_1.createUriMap)();
    const file2ProjectInfo = new Map();
    server.fileWatcher.onDidChangeWatchedFiles(({ changes }) => {
        for (const change of changes) {
            const changeUri = vscode_uri_1.URI.parse(change.uri);
            if (tsconfigProjects.has(changeUri)) {
                tsconfigProjects.get(changeUri).dispose();
                tsconfigProjects.delete(changeUri);
                file2ProjectInfo.clear();
            }
        }
    });
    let simpleLs;
    return server.initialize(params, {
        setup() { },
        async getLanguageService(uri) {
            if (uri.scheme === 'file' && options.typescript.tsserverRequestCommand) {
                const fileName = uri.fsPath.replace(/\\/g, '/');
                let projectInfoPromise = file2ProjectInfo.get(fileName);
                if (!projectInfoPromise) {
                    projectInfoPromise = sendTsRequest(ts.server.protocol.CommandTypes.ProjectInfo, {
                        file: fileName,
                        needFileNameList: false,
                    });
                    file2ProjectInfo.set(fileName, projectInfoPromise);
                }
                const projectInfo = await projectInfoPromise;
                if (projectInfo) {
                    const { configFileName } = projectInfo;
                    let ls = tsconfigProjects.get(vscode_uri_1.URI.file(configFileName));
                    if (!ls) {
                        ls = createLs(server, configFileName);
                        tsconfigProjects.set(vscode_uri_1.URI.file(configFileName), ls);
                    }
                    return ls;
                }
            }
            return simpleLs ??= createLs(server, undefined);
        },
        getExistingLanguageServices() {
            return Promise.all([
                ...tsconfigProjects.values(),
                simpleLs,
            ].filter(promise => !!promise));
        },
        reload() {
            for (const ls of [
                ...tsconfigProjects.values(),
                simpleLs,
            ]) {
                ls?.dispose();
            }
            tsconfigProjects.clear();
            simpleLs = undefined;
        },
    }, (0, language_service_1.getHybridModeLanguageServicePlugins)(ts, options.typescript.tsserverRequestCommand ? {
        collectExtractProps(...args) {
            return sendTsRequest('vue:collectExtractProps', args);
        },
        getComponentDirectives(...args) {
            return sendTsRequest('vue:getComponentDirectives', args);
        },
        getComponentEvents(...args) {
            return sendTsRequest('vue:getComponentEvents', args);
        },
        getComponentNames(...args) {
            return sendTsRequest('vue:getComponentNames', args);
        },
        getComponentProps(...args) {
            return sendTsRequest('vue:getComponentProps', args);
        },
        getElementAttrs(...args) {
            return sendTsRequest('vue:getElementAttrs', args);
        },
        getElementNames(...args) {
            return sendTsRequest('vue:getElementNames', args);
        },
        getImportPathForFile(...args) {
            return sendTsRequest('vue:getImportPathForFile', args);
        },
        getPropertiesAtLocation(...args) {
            return sendTsRequest('vue:getPropertiesAtLocation', args);
        },
        getDocumentHighlights(fileName, position) {
            return sendTsRequest('documentHighlights-full', // internal command
            {
                file: fileName,
                ...{ position },
                filesToSearch: [fileName],
            });
        },
        async getQuickInfoAtPosition(fileName, { line, character }) {
            const result = await sendTsRequest(ts.server.protocol.CommandTypes.Quickinfo, {
                file: fileName,
                line: line + 1,
                offset: character + 1,
            });
            return ts.displayPartsToString(result?.displayParts ?? []);
        },
    } : undefined));
    function sendTsRequest(command, args) {
        return connection.sendRequest(options.typescript.tsserverRequestCommand, [command, args]);
    }
    function createLs(server, tsconfig) {
        const commonLine = tsconfig
            ? (0, language_core_1.createParsedCommandLine)(ts, ts.sys, tsconfig)
            : {
                options: ts.getDefaultCompilerOptions(),
                vueOptions: (0, language_core_1.getDefaultCompilerOptions)(),
            };
        const language = (0, language_core_1.createLanguage)([
            {
                getLanguageId: uri => server.documents.get(uri)?.languageId,
            },
            (0, language_core_1.createVueLanguagePlugin)(ts, commonLine.options, commonLine.vueOptions, uri => uri.fsPath.replace(/\\/g, '/')),
        ], (0, language_service_1.createUriMap)(), uri => {
            const document = server.documents.get(uri);
            if (document) {
                language.scripts.set(uri, document.getSnapshot(), document.languageId);
            }
            else {
                language.scripts.delete(uri);
            }
        });
        return (0, language_service_1.createLanguageService)(language, server.languageServicePlugins, (0, simpleProject_1.createLanguageServiceEnvironment)(server, [...server.workspaceFolders.all]), { vue: { compilerOptions: commonLine.vueOptions } });
    }
});
connection.onInitialized(server.initialized);
connection.onShutdown(server.shutdown);
//# sourceMappingURL=node.js.map